# -*- coding: utf-8 -*-
"""Workspace/project-oriented tmux/git personal assistant.

Working on multiple git repositories and juggling tmux sessions can be tedious.
`mx` tries to help with that.
"""
__package__ = 'mx'
__license__ = 'MIT'
__version__ = '0.2.17'
__author__ = __maintainer__ = 'Rafael Bodill'
__email__ = 'justrafi@gmail'
